// Code written by R. Soulard

// depends on 'util.sci' for FFT
function y = gaussian(x,sigm)
  // Periodic Gaussian low-pass filtering.
  // x : input image
  // sigm : value of sigma
  // y : filtered image
  
  // Handle hypermatrices (multivalued images)
  // in a marginal way:
  if(length(size(x))>2) then
    y = x;
    for i=1:size(x,3),
      y(:,:,i) = gaussian(x(:,:,i),sigm);
    end
  // Scalar case:
  else
    if(sigm==0) then
      y = x;
    else
      [M,N]=size(x);
      x1 = [0:ceil(M/2)-1 , -floor(M/2):-1]' * ones(1,N);
      x2 = ones(M,1) * [0:ceil(N/2)-1 , -floor(N/2):-1];
      N2 = x1.^2+x2.^2;
      h = (1/(2*%pi*sigm*sigm)) * exp( -N2/(2*sigm*sigm) );
      y = real(rs_ifft(rs_fft(x).*rs_fft(h)));
    end
  end
endfunction
