// Code for 3-valued oscillation's ellipse parameters
// [Lilly, J. M. Modulated oscillations in three dimensions 
//               IEEE Trans. Signal Process., 2011, 59, 5930-5943]

// Scilab code written by R. Soulard in 2014
// for the paper:
// [Soulard & Carre, ELLIPTICAL MONOGENIC REPRESENTATION OF COLOR IMAGES
//                   AND LOCAL FREQUENCY ANALYSIS, ICIP 2015 ]

function [ka,la,phi,alph,bet,thet]=lilly_ellipse(varargin)
  // input :   x , y , z , xh , yh , zh   (MxN   matrices)
  //   or :   cimg(x,y,z) , cimg(xh,yh,zh) (MxNx3 matrices)
  if(argn(2)==6) then
    x  = varargin(1);
    y  = varargin(2);
    z  = varargin(3);
    xh = varargin(4);
    yh = varargin(5);
    zh = varargin(6);
  elseif(argn(2)==2) then
    xyz = varargin(1);
    xyzh= varargin(2);
    x = xyz(:,:,1);
    y = xyz(:,:,2);
    z = xyz(:,:,3);
    xh= xyzh(:,:,1);
    yh= xyzh(:,:,2);
    zh= xyzh(:,:,3);
    clear xyz xyzh
  else
    printf("lilly_ellipse() : bad argument!\n");
  end
    
  amp = sqrt(x.^2+y.^2+z.^2+xh.^2+yh.^2+zh.^2);
  ka = (1/sqrt(2)) * amp;
  // normal vector :
  nx =  yh.*z - zh.*y ; // normal vector
  ny =-(xh.*z - zh.*x); // normal vector
  nz =  xh.*y - yh.*x ; // normal vector
  [nxy,alph] = modarg( -ny , nx );
  [namp,bet] = modarg( nz , nxy );
  la = sqrt( 1 - 4*( (namp.^2)./(amp.^4) ));
  ca = cos(alph);
  sa = sin(alph);
  cb = cos(bet);
  sb = sin(bet);
  v21 =      ca.*(x+%i*xh) +     sa.*(y+%i*yh);
  v22 = -sa.*cb.*(x+%i*xh) + ca.*cb.*(y+%i*yh) + sb.*(z+%i*zh);
  [tmp,phi1] = modarg(v21+%i*v22);
  [tmp,phi2] = modarg(v21-%i*v22);
  phi = 0.5*(phi1+phi2);
  thet =anglewrap( 0.5*(phi1-phi2) , '[-pi;pi]');
  idx = thet<0 ;
  thet(idx) = thet(idx)+%pi;
  phi(idx) = phi(idx)+%pi;
  phi = anglewrap( phi, '[0;2pi]');
endfunction

function [x,y,z,xh,yh,zh]=ililly_ellipse(ka,la,phi,alph,bet,thet)
  a = ka.*sqrt(1+la);
  b = ka.*sqrt(1-la);
  ca = cos(alph);
  sa = sin(alph);
  cb = cos(bet);
  sb = sin(bet);
  ct = cos(thet);
  st = sin(thet);
  v1 = a.*(ca.*ct - sa.*cb.*st) + %i*b.*(ca.*st + sa.*cb.*ct);
  v2 = a.*(sa.*ct + ca.*cb.*st) + %i*b.*(sa.*st - ca.*cb.*ct);
  v3 = a.*sb.*st - %i*b.*sb.*ct;
  xp = v1.*exp(%i*phi);
  yp = v2.*exp(%i*phi);
  zp = v3.*exp(%i*phi);
  x=real(xp); xh=imag(xp);
  y=real(yp); yh=imag(yp);
  z=real(zp); zh=imag(zp);
endfunction
