// ================================================================== //
// R. Soulard and P. Carré 2015
// Elliptical monogenic representation
// => Color local frequency
// ================================================================== //

clear
exec('util.sci');
exec('SQF.sci');
exec('gaussian.sci');
exec('tensor.sci');
exec('lilly_ellipse.sci');
exec('inst_freq_2D.sci');
exec('rsimg.sci');
stacksize('max');

// ================================================================== //
// Synthetic test image 
// Make a test image made of 9 color waves with fixed known parameters.
// ================================================================== //
function im = colorwav(siz,nu,ori,AR,AG,AB,phiR,phiG,phiB)
  N = siz;
  x = ((0:N-1)-floor(N/2));
  xx = x'*ones(1,N);
  yy = ones(N,1)*x;
  dis = sqrt(xx.^2+yy.^2);
  ph = anglewrap(  nu*(cos(ori)*xx+sin(ori)*yy) , '[0;2pi]' );
  im = cimg(AR*cos(ph+phiR),AG*cos(ph+phiG),AB*cos(ph+phiB));
  im = im/max(abs(im));
endfunction
k=61;
//              size,freq , ori , AR , AG , AB , phiR, phiG, phiB
im = [ colorwav( k , 1.8 , 0.3 ,1.00,0.30,0.45, 0   , 0.1 , 0.2  ) ,...
       colorwav( k , 1.9 , 0.4 ,1.00,1.00,0.36,-0.1 ,-0.1 ,-0.3  ) ,...
       colorwav( k , 2.5 , 0.1 ,1.65,1.65,1.65, %pi , 0   , 0    ) ;...
       colorwav( k , 0.5 , 1.6 ,0.70,0.18,0.36,-2.0 ,-0.3 ,-0.1  ) ,...
       colorwav( k , 1.1 ,-0.4 ,0.70,0.18,0.36,-2.0 ,-0.3 ,-0.1  ) ,...
       colorwav( k , 1.5 , 0.0 ,0.70,0.18,0.36,-2.0 ,-0.3 ,-0.1  ) ;...
       colorwav( k , 0.35, 0.6 ,0.60,0.05,0.36, 1.0 , 1.3 ,-0.1  ) ,...
       colorwav( k , 0.4 ,-0.8 ,0.55,0.30,0.11, 0.3 ,-0.4 , 0.1  ) ,...
       colorwav( k , 0.8 , 1.5 ,1.00,1.00,1.00, 0.0 ,-%pi , 0.0  ) ];
im = 0.5+0.5*im/max(abs(im)); // centering & normalizing
// Theoretical nu values:
t=ones(k,k);
th_nu = [1.8*t,1.9*t,2.5*t ; 0.5*t,1.1*t,1.5*t ; 0.35*t,0.4*t,0.8*t];

nu_min = 0.2; 
nu_max = 2.5;
median_filt_width = 0;
HISTPLOT = %F;
// ================================================================== //


// ================================================================== //
// Natural test image 
// ================================================================== //
im=double(imread('carpet400.png'))/255; 
th_nu = %pi*ones(size(im,1),size(im,2)); // (unused)

nu_min = 0.3; 
nu_max = 1.5;
median_filt_width = 11;
HISTPLOT = %T;
// ================================================================== //







// ================================================================== //
// Extract local frequency:
// ================================================================== //

// *** Isotropic filtering in the band [ nu_min , nu_max ] 
//     and Riesz transform:
[s,sx,sy,H,Hx,Hy] = SQF( im , nu_min , nu_max );

// *** Get local frequency: 
gauss_sigma = 1;     // smoothing width for local orientation estimation
[n,lambdaP,coher,ori] = tensor(sx,sy,gauss_sigma);     // struct. tensor
sh = ( sx .* cimg(cos(ori)) + sy .* cimg(sin(ori)) );  // directional RT
[kap,lam, phaz ,a1,a2,a3] = lilly_ellipse( s , sh );   // ellipse params
nu = inst_freq_2D( phaz , ori );     // differentiate phase to get freq.

// *** Optional median filtering:
k=floor(median_filt_width/2);
nu2 = nu;
[M,N]=size(im(:,:,1));
for i=1:M,
  for j=1:N,
    nu_patch = nu(max(1,i-k):min(M,i+k),max(1,j-k):min(N,j+k));
    nu2(i,j) = median(nu_patch(:));
  end
end
nu = nu2;

// ================================================================== //
// Display results:
// ================================================================== //

imshow([im,cimg(  ([nu,th_nu]-nu_min)/(nu_max-nu_min)  )]);
scf(1);clf(1); ff=gcf();set(ff,"color_map",graycolormap(64));
sc1=1:floor(M/20):M; // subsample the mesh for publication convenience
sc2=1:floor(N/30):N;
mesh(sc1,sc2,nu(sc1,sc2)'); a=gca();a=a.children(1);a.color_flag=3;a.color_mode=-1;

if(HISTPLOT) then
  scf(2); clf(2); histplot(nu_min:(nu_max-nu_min)/64:nu_max,nu);
end

// ================================================================== //
disp('end of script.');
// ================================================================== //
