// extraction of structure tensor features
// from the gradient of an image 
// [needs 'util.sci' and 'gaussian.sci']
// Can be used with a Riesz transform as well,
// see [Soulard, Carre & Fernandez, Vector extension of monogenic 
//      wavelets for geometric representation of color images 
//      IEEE Trans. Image Process., 2013, 22, 1070-1083 ]
// Code written by R. Soulard

function [N,lambdaP,coher,orient]=tensor(Ix,Iy,sigm)
  // -------------------------------------------------------------- //
  // Ix,Iy: marginal gradient
  // sigm: width of gaussian filtering, if 0 : no filtering
  // N2: squared norm of the gradient
  // lambdaP: higher eigen value of tensor
  // coher: coherency in [0;1]
  // orient: orientation of eigenvector for higher eigenvalue
  // -------------------------------------------------------------- //
  // tensor components ( T=[a,b;b,c] ):
  a = csum(Ix.^2);
  b = csum(Ix.*Iy);
  c = csum(Iy.^2);
  // Smoothing of tensor components:
  if(sigm>0) then
    a = gaussian(a,sigm);
    b = gaussian(b,sigm);
    c = gaussian(c,sigm);
  end
  N2 = a+c;
  N = sqrt(N2);
  [r,twotheta]=modarg( a-c , 2*b );
  orient = twotheta/2;
  lambdaP = 0.5*(N2+r);
//  lambdaM = 0.5*(N2-r);
  mask = double(N2==0);
  coher = r.*(1-mask)./(N2+mask);
endfunction
